==> htsct-0.0.51-py3-none-any.whl: htsct/core/calculator.py <==
from datetime import datetime
import yaml
import ase.io
from htsct.utils.tools import Pipe
from htsct.io.vaspIO import writeIncar, writeKpoints, writeOptcell, writePotcar
from pathlib import Path
from htsct.io import yacif
import wrapt


def taskLogDecorator(task_type):
    """
    类装饰器，用于记录类中函数执行时间
    :param task_type:工作类型
    """

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        st = datetime.now()
        result = wrapped(*args, **kwargs)
        et = datetime.now()
        time_span = et - st
        logPath = Path(instance.logPath)
        logPath.parent.mkdir(parents=True, exist_ok=True)
        with open(logPath, 'a+') as fd:
            yaml.dump(
                {st: {'path': str(instance.root), "task type": task_type, "time used": str(time_span)}},
                fd)
        return result

    return wrapper


class Calculator:
    def __init__(self, root, logPath):
        self.root = Path(root).absolute()
        self.structGamDir = self.root / "gam"
        self.structDir = self.root / "struct"
        self.scfDir = self.root / "scf"
        self.bandDir = self.root / "band"
        self.logPath = Path(logPath)

    def checkPOSCAR(self, src: Path, target: Path):
        POSCAR = src / "POSCAR"
        target.mkdir(parents=True, exist_ok=True)
        for file in POSCAR.parent.iterdir():
            if ".cif" in file.name:
                ase.io.read(file).write(POSCAR, "vasp")
            elif ".yacif" in file.name:
                yacif.read(file).write(POSCAR, "vasp")
        if not POSCAR.exists():
            raise Exception("无法找到输入结构文件")
        (src / "POSCAR").replace(target / "POSCAR")

    @taskLogDecorator("struct_gam")
    def structGam(self, command, optcell=((1, 0, 0), (0, 0, 0), (0, 0, 0)), subdivisions=(1, 1, 1), **kwargs):
        """Gam Structure optimization calculator.
        """
        writeIncar(self.structGamDir, **kwargs)  # 生成INCAR
        self.checkPOSCAR(self.root, self.structGamDir)  # 生成POSCAR文件
        writeOptcell(self.structGamDir, optcell[0], optcell[1], optcell[2])  # 生成OPTCELL
        writeKpoints(self.structGamDir, subdivisions)  # 生成KPOINTS
        writePotcar(self.structGamDir)
        writePotcar(self.structGamDir)
        self.runCommand(command, self.structGamDir)

    @taskLogDecorator("struct")
    def struct(self, command, optcell=((1, 0, 0), (0, 0, 0), (0, 0, 0)), subdivisions=(6, 1, 1), **kwargs):
        """Structure optimization calculator.
        """
        writeIncar(self.structDir, **kwargs)  # 生成INCAR
        writeKpoints(self.structDir, subdivisions)  # 生成KPOINTS
        writePotcar(self.structDir)
        writeOptcell(self.structDir, optcell)  # 生成OPTCELL
        (self.structGamDir / "CONTCAR").replace(self.structDir / "POSCAR")  # 生成POSCAR
        self.runCommand(command, self.structDir)

    @taskLogDecorator("scf")
    def scf(self, command, optcell=((1, 0, 0), (0, 0, 0), (0, 0, 0)), subdivisions=(6, 1, 1), **kwargs):
        """Scf calculator.
        """
        writeIncar(self.scfDir, **kwargs)
        writeKpoints(self.scfDir, subdivisions)
        writePotcar(self.scfDir)
        writeOptcell(self.scfDir, optcell)
        (self.structDir / "CONTCAR").replace(self.scfDir / "POSCAR")
        self.runCommand(command, self.scfDir)

    @taskLogDecorator("band")
    def band(self, command, kpaths=None, density=0, vaspkit="", **kwargs):
        """Band calculator.
        """
        writeIncar(self.bandDir, **kwargs)
        writePotcar(self.bandDir)
        writeKpoints(self.bandDir, mode="BandStructure", kpaths=kpaths, density=density, vaspkit=vaspkit)
        (self.scfDir / "CONTCAR").replace(self.bandDir / "POSCAR")
        (self.scfDir / "CHGCAR").replace(self.bandDir / "CHGCAR")
        (self.scfDir / "WAVECAR").replace(self.bandDir / "WAVECAR")
        self.runCommand(command, self.bandDir)

    @staticmethod
    def runCommand(command, directory: Path):
        _commands = ["local NP\n",
                     "local OPTIND\n",
                     'NP=$(cat "$PBS_NODEFILE" | wc -l)\n',
                     'cat "$PBS_NODEFILE" | sort | uniq | tee /tmp/nodes.$$ | wc -l\n',
                     'cat "$PBS_NODEFILE" >/tmp/nodefile.$$\n',
                     f'mpirun -genv I_MPI_DEVICE ssm -machinefile /tmp/nodefile.$$ -n "$NP" {command}\n',
                     'rm - rf / tmp / nodefile.$$\n',
                     'rm - rf / tmp / nodes.$$\n'
                     ]
        cmds = "".join(_commands)
        stdout = Pipe(cmds, directory).stdout
        with open(directory / "vasp.out", "w+") as fd:
            fd.write(stdout)

==> htsct-0.0.53-py3-none-any.whl: htsct/core/calculator.py <==
from datetime import datetime
import yaml
import ase.io
from htsct.utils.tools import Pipe
from htsct.io.vaspIO import writeIncar, writeKpoints, writeOptcell, writePotcar
from pathlib import Path
from htsct.io import yacif
import wrapt


def taskLogDecorator(task_type):
    """
    类装饰器，用于记录类中函数执行时间
    :param task_type:工作类型
    """

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        st = datetime.now()
        result = wrapped(*args, **kwargs)
        et = datetime.now()
        time_span = et - st
        logPath = Path(instance.logPath)
        logPath.parent.mkdir(parents=True, exist_ok=True)
        with open(logPath, 'a+') as fd:
            yaml.dump(
                {st: {'path': str(instance.root), "task type": task_type, "time used": str(time_span)}},
                fd)
        return result

    return wrapper


class Calculator:
    def __init__(self, root, logPath):
        self.root = Path(root).absolute()
        self.structGamDir = self.root / "gam"
        self.structDir = self.root / "struct"
        self.scfDir = self.root / "scf"
        self.bandDir = self.root / "band"
        self.logPath = Path(logPath)

    def checkPOSCAR(self, src: Path, target: Path):
        POSCAR = src / "POSCAR.ase"
        target.mkdir(parents=True, exist_ok=True)
        for file in POSCAR.parent.iterdir():
            if ".cif" in file.name:
                ase.io.read(file).write(POSCAR, "vasp")
            elif ".yacif" in file.name:
                yacif.read(file).write(POSCAR, "vasp")
        if not POSCAR.exists():
            raise Exception("无法找到输入结构文件")
        with open(POSCAR) as fd:
            lines = fd.readlines()
        symbol = lines[5].split()
        number = lines[6].split()
        positions = lines[8:]
        d = {}
        for s, n, p in zip(symbol, number, positions):
            n = int(n)
            if s not in d.keys():
                d.update({s: [n, [p, ]]})
            else:
                d[s][1].append(p)
                d[s][0] = d[s][0] + 1
        _symbol = []
        _number = []
        _positions = []
        for k, v in d.items():
            _symbol.append(k)
            _number.append(str(v[0]))
            _positions.extend(v[1])
        name = "".join([f"{s}{n}" for s, n in zip(_symbol, _number)])
        content = f"{name}\n{''.join(lines[1:5])}{' ' + ' '.join(_symbol)}\n{'  ' + ' '.join(_number)}\n{lines[7]}{''.join(_positions)}"
        with open(target / "POSCAR", "w+") as fd:
            fd.write(content)

    @taskLogDecorator("struct_gam")
    def structGam(self, command, optcell=((1, 0, 0), (0, 0, 0), (0, 0, 0)), subdivisions=(1, 1, 1), **kwargs):
        """Gam Structure optimization calculator.
        """
        writeIncar(self.structGamDir, **kwargs)  # 生成INCAR
        self.checkPOSCAR(self.root, self.structGamDir)  # 生成POSCAR文件
        writeOptcell(self.structGamDir, optcell[0], optcell[1], optcell[2])  # 生成OPTCELL
        writeKpoints(self.structGamDir, subdivisions)  # 生成KPOINTS
        writePotcar(self.structGamDir)
        raise Exception(f"{(self.structGamDir / 'KPOINTS').exists()}")
        self.runCommand(command, self.structGamDir)

    @taskLogDecorator("struct")
    def struct(self, command, optcell=((1, 0, 0), (0, 0, 0), (0, 0, 0)), subdivisions=(6, 1, 1), **kwargs):
        """Structure optimization calculator.
        """
        writeIncar(self.structDir, **kwargs)  # 生成INCAR
        writeKpoints(self.structDir, subdivisions)  # 生成KPOINTS
        writePotcar(self.structDir)
        writeOptcell(self.structDir, optcell)  # 生成OPTCELL
        (self.structGamDir / "CONTCAR").replace(self.structDir / "POSCAR")  # 生成POSCAR
        self.runCommand(command, self.structDir)

    @taskLogDecorator("scf")
    def scf(self, command, optcell=((1, 0, 0), (0, 0, 0), (0, 0, 0)), subdivisions=(6, 1, 1), **kwargs):
        """Scf calculator.
        """
        writeIncar(self.scfDir, **kwargs)
        writeKpoints(self.scfDir, subdivisions)
        writePotcar(self.scfDir)
        writeOptcell(self.scfDir, optcell)
        (self.structDir / "CONTCAR").replace(self.scfDir / "POSCAR")
        self.runCommand(command, self.scfDir)

    @taskLogDecorator("band")
    def band(self, command, kpaths=None, density=0, vaspkit="", **kwargs):
        """Band calculator.
        """
        writeIncar(self.bandDir, **kwargs)
        writePotcar(self.bandDir)
        writeKpoints(self.bandDir, mode="BandStructure", kpaths=kpaths, density=density, vaspkit=vaspkit)
        (self.scfDir / "CONTCAR").replace(self.bandDir / "POSCAR")
        (self.scfDir / "CHGCAR").replace(self.bandDir / "CHGCAR")
        (self.scfDir / "WAVECAR").replace(self.bandDir / "WAVECAR")
        self.runCommand(command, self.bandDir)

    @staticmethod
    def runCommand(command, directory: Path):
        _commands = ["local NP\n",
                     "local OPTIND\n",
                     'NP=$(cat "$PBS_NODEFILE" | wc -l)\n',
                     'cat "$PBS_NODEFILE" | sort | uniq | tee /tmp/nodes.$$ | wc -l\n',
                     'cat "$PBS_NODEFILE" >/tmp/nodefile.$$\n',
                     f'mpirun -genv I_MPI_DEVICE ssm -machinefile /tmp/nodefile.$$ -n "$NP" {command}\n',
                     'rm - rf / tmp / nodefile.$$\n',
                     'rm - rf / tmp / nodes.$$\n'
                     ]
        cmds = "".join(_commands)
        stdout = Pipe(cmds, directory).stdout
        with open(directory / "vasp.out", "w+") as fd:
            fd.write(stdout)
